From e0f9ae10b526f5a9eefedee6700d12e07b964637 Mon Sep 17 00:00:00 2001
From: Adrien Destugues <pulkomandy@pulkomandy.tk>
Date: Tue, 1 Mar 2022 20:08:25 +0100
Subject: [PATCH] sdhci_pci: enable timeout interrupt.

Supposedly we should get a "command complete" interrupt even on
timeouts, but we don't. So enable the separate interrupt for timeouts as
well, and hopefully we can handle them both together.

FreeBSD just enables all interrupt bits, and seems to be working fine.

12	11	src/add-ons/kernel/busses/mmc/sdhci_pci.cpp
1	1	src/add-ons/kernel/busses/mmc/sdhci_pci.h

diff --git a/src/add-ons/kernel/busses/mmc/sdhci_pci.cpp b/src/add-ons/kernel/busses/mmc/sdhci_pci.cpp
index 437e4ff216..688584465c 100644
--- a/src/add-ons/kernel/busses/mmc/sdhci_pci.cpp
+++ b/src/add-ons/kernel/busses/mmc/sdhci_pci.cpp
@@ -127,13 +127,13 @@ SdhciBus::SdhciBus(struct registers* registers, uint8_t irq)
 
 	// Finally, configure some useful interrupts
 	EnableInterrupts(SDHCI_INT_CMD_CMP | SDHCI_INT_CARD_REM
-		| SDHCI_INT_TRANS_CMP);
+		| SDHCI_INT_TRANS_CMP | SDHCI_INT_TIMEOUT);
 
 	// We want to see the error bits in the status register, but not have an
 	// interrupt trigger on them (we get a "command complete" interrupt on
 	// errors already)
 	fRegisters->interrupt_status_enable |= SDHCI_INT_ERROR
-		| SDHCI_INT_TIMEOUT | SDHCI_INT_CRC | SDHCI_INT_INDEX
+		| SDHCI_INT_CRC | SDHCI_INT_INDEX
 		| SDHCI_INT_BUS_POWER | SDHCI_INT_END_BIT;
 }
 
@@ -271,15 +271,6 @@ SdhciBus::ExecuteCommand(uint8_t command, uint32_t argument, uint32_t* response)
 
 	if (fCommandResult & SDHCI_INT_ERROR) {
 		fRegisters->interrupt_status |= fCommandResult;
-		if (fCommandResult & SDHCI_INT_TIMEOUT) {
-			ERROR("Command execution timed out\n");
-			if (fRegisters->present_state.CommandInhibit()) {
-				TRACE("Command line is still busy, clearing it\n");
-				// Clear the stall
-				fRegisters->software_reset.ResetCommandLine();
-			}
-			return B_TIMED_OUT;
-		}
 		if (fCommandResult & SDHCI_INT_CRC) {
 			ERROR("CRC error\n");
 			return B_BAD_VALUE;
@@ -290,6 +281,16 @@ SdhciBus::ExecuteCommand(uint8_t command, uint32_t argument, uint32_t* response)
 		return B_ERROR;
 	}
 
+	if (fCommandResult & SDHCI_INT_TIMEOUT) {
+		ERROR("Command execution timed out\n");
+		if (fRegisters->present_state.CommandInhibit()) {
+			TRACE("Command line is still busy, clearing it\n");
+			// Clear the stall
+			fRegisters->software_reset.ResetCommandLine();
+		}
+		return B_TIMED_OUT;
+	}
+
 	if (fRegisters->present_state.CommandInhibit()) {
 		TRACE("Command execution failed, card stalled\n");
 		// Clear the stall
diff --git a/src/add-ons/kernel/busses/mmc/sdhci_pci.h b/src/add-ons/kernel/busses/mmc/sdhci_pci.h
index 3a04de6df4..c9a4081e80 100644
--- a/src/add-ons/kernel/busses/mmc/sdhci_pci.h
+++ b/src/add-ons/kernel/busses/mmc/sdhci_pci.h
@@ -194,7 +194,7 @@ class SoftwareReset {
 #define	 SDHCI_INT_CMD_ERROR_MASK	(SDHCI_INT_TIMEOUT | \
 		SDHCI_INT_CRC | SDHCI_INT_END_BIT | SDHCI_INT_INDEX)
 
-#define SDHCI_INT_CMD_MASK 	(SDHCI_INT_CMD_CMP | SDHCI_INT_CMD_ERROR_MASK)
+#define SDHCI_INT_CMD_MASK 	(SDHCI_INT_CMD_CMP | SDHCI_INT_CMD_ERROR_MASK | SDHCI_INT_TIMEOUT)
 
 // #pragma mark -
 class Capabilities
-- 
2.11.0

